import Knex = require('knex');
import Koa = require('koa');
import Bluebird = require('bluebird');

type Upload = {
    pathname: string,
    filename: string
}

interface PropApi {
    (): void;
}

interface PropRpc {
    (): void;
}

interface PropSocket {
    (option: { event: string }): void
}


interface PropAction {
    (option: { auth: boolean, login: boolean }): void
}

// type PropApi = () => Function;
// type PropRpc = () => Function;
// type PropAction = (option: { event: string }) => Function;
// type PropSocket = (option: { auth: boolean, login: boolean }) => Function;


declare var jinghuan: {
    readonly JH_ROOT: string;
    readonly APP_PATH: string;
    readonly ROOT_PATH: string;
    readonly modules: string[];
    readonly modulePaths: object;
    readonly mode: string;
    readonly source: string;
    readonly env: string;
    readonly workers: string;
    readonly PORT: string;
    readonly HOST: string;
    readonly paths: string[];
    readonly process_id: number;
    readonly watcher: boolean;
    readonly version: string;
    readonly Controller: JinghuanController;
    readonly AdminController: JinghuanController;
    readonly ApiController: JinghuanController;
    readonly app: Koa;
    readonly logger: {
        log(...args: any[]): void;
        debug(...args: any[]): void;
        info(...args: any[]): void;
        debug(...args: any[]): void;
        warn(...args: any[]): void;
    }
};


interface Application extends Koa {
    request: Request;
    response: Response;
}

interface JinghuanDb extends Knex.QueryBuilder {
    sql(sql: string): JinghuanDb
}

interface Request extends Koa.Request {
}

interface Response extends Koa.Response {
}

interface JinghuanContext extends Koa.Context {
    readonly module: string;
    readonly controller: string;
    readonly action: string;
    readonly userAgent: string;
    readonly isGet: boolean;
    readonly isPost: boolean;
    readonly body: any;

    referer(onlyHost: boolean): string;

    referrer(onlyHost: boolean): string;

    isMethod(method: string): boolean;

    isAjax(method: string): boolean;

    expires(time: any): any

    param(): object;

    param(name: string): object;

    post(): object;

    post(value: object): this;

    file(): object;

    file(data: object): this;

    file(name: string): any;

    file(name: string, value: any): this;

    download(filepath: string, filename?: string): void;

    db(table: string): JinghuanDb;

    db(type: symbol, table: string): JinghuanDb;

    db<T>(transactionScope: (trx: Knex.Transaction) => Promise<T> | Bluebird<T> | void): Bluebird<T>;

    sql(name: string): JinghuanDb;

    sql(type: symbol, name: string): JinghuanDb;

    upload(file: string, dir: string, filename: string, filter?: Array): Upload
}

interface JinghuanController {
    readonly ctx: JinghuanContext;
    readonly userAgent: string;
    readonly isGet: boolean;
    readonly isPost: boolean;
    readonly body: any;

    referer(onlyHost: boolean): string;

    referrer(onlyHost: boolean): string;

    isMethod(method: string): boolean;

    isAjax(method: string): boolean;

    expires(time: any): any

    param(): object;

    param(name: string): object;

    post(): object;

    post(value: object): this;

    file(): object;

    file(data: object): this;

    file(name: string): any;

    file(name: string, value: any): this;

    download(filepath: string, filename?: string): void;

    db(table: string): JinghuanDb;

    db(type: symbol, table: string): JinghuanDb;

    db<T>(transactionScope: (trx: Knex.Transaction) => Promise<T> | Bluebird<T> | void): Bluebird<T>;

    sql(name: string): JinghuanDb;

    sql(type: symbol, name: string): JinghuanDb;

    upload(file: string, dir: string, filename: string, filter?: Array): Upload
}

